#include "XGS_PIC_SYSTEM_V010.h"
#include "XGS_PIC_GAMEPAD_DRV_V010.h"
#include "XGS_PIC_GFX_DRV_V010.h"
#include "XGS_PIC_NTSC_160_192_2_V010.h"
#include "XGS_PIC_SOUND_DRV_V010.h"

// Sets the side length of the ball
#define BALL	4

// Constants to define the ball movement states
#define BALL_UP			 0
#define BALL_DOWN		 1
#define BALL_LEFT		 0
#define BALL_RIGHT		 1

// Global variables needed for the graphics drivers
unsigned char g_VRAMBuffer[VRAM_BUF_SIZE] __attribute((far));
unsigned short g_PaletteMap[PALETTE_MAP_SIZE];
unsigned char g_Palettes[MIN_PALETTE_SIZE];

// Draws a ball on the screen with (x,y) as the top left corner with the
// specified color in the palette
void drawBall(unsigned int x, unsigned int y, unsigned int colorIndex) {
	int i, j;
	for (i = x; i < x+BALL; i++)
		for (j = y; j < y+BALL; j++)
			GFX_Plot_2BPP(i, j, colorIndex, g_VRAMBuffer);
}

void resetBall(unsigned int *ball_x, unsigned int *ball_y) {
	(*ball_x) = SCREEN_WIDTH/2-BALL/2;
	(*ball_y) = SCREEN_HEIGHT/2-BALL/2;
}

int main(void) {
	// INITIALIZATION
	// Setting up the processor for graphics mode
	SYS_ConfigureClock(FCY_NTSC);
	
	// Setting up the graphics drivers
	GFX_InitBitmap(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, g_VRAMBuffer, g_Palettes, g_PaletteMap);
	
	memset(g_PaletteMap, 0, sizeof(g_PaletteMap));
	memset(g_Palettes, NTSC_BLACK, sizeof(g_Palettes));

	g_Palettes[1] = NTSC_WHITE;

	// Setting up the gamepad drivers
	Gamepad_Init();
	// Setting up the sound drivers
	SND_Init();

	// Creating location variables for the onscreen objects and
	// setting their starting states

	// Ball's location variables
	unsigned int ball_x;
	unsigned int ball_y;

	// Ball's state variables
	unsigned int state_x = BALL_UP;
	unsigned int state_y = BALL_LEFT;

	// Setting up the first frame
	resetBall(&ball_x, &ball_y);
	drawBall(ball_x, ball_y, 1);

	// Starting the graphics engine
	GFX_StartDrawing(SCREEN_TYPE_NTSC);

	// Starting the game loop
	while (1) {
		WAIT_FOR_VSYNC_START();

		// Ball movment
		if (state_y == BALL_UP) 
			if (ball_y == 0)
				state_y = BALL_DOWN;
			else
				ball_y--;
		else
			if (ball_y+BALL == SCREEN_HEIGHT)
				state_y = BALL_UP;
			else
				ball_y++;

		if (state_x == BALL_LEFT)
			if (ball_x == 0)
				state_x = BALL_RIGHT;
			else
				ball_x--;
		else
			if (ball_x == SCREEN_WIDTH-BALL)
				state_x = BALL_LEFT;
			else
				ball_x++;

		// Redrawing the screen
		GFX_FillScreen_2BPP(0, g_VRAMBuffer);
		drawBall(ball_x, ball_y, 1);

		WAIT_FOR_VSYNC_END();
	}
	return 0;
}